package com.care.bms.mqtt;

import com.care.bms.service.BmsEventOrderService;
import com.care.common.entity.CareDevice;
import com.care.common.service.*;
import com.care.common.util.CommonConfUtil;
import org.springframework.util.StringUtils;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

public class MqttTool {
    private static Map<String, MqttDataConnector> mqttDataConnectorMap = new HashMap<>();
    public static void init(CareDeviceService careDeviceService,
                            BmsEventOrderService bmsEventOrderService,
                            CareEventOrderService careEventOrderService,
                            CareHouseService careHouseService,
                            CareOrganizationService careOrganizationService,
                            CareStationService careStationService) {
        List<CareDevice> listCareDevice = careDeviceService.list();
        for (CareDevice careDevice : listCareDevice) {
            if (StringUtils.isEmpty(careDevice.getTopic()) || StringUtils.isEmpty(careDevice.getClientId())) {
                continue;
            }
            MqttDataConnector xbMqttDataConnector = new MqttDataConnector();
            Map<String, String> configParams = new HashMap<>();
            String host = "ws://" + CommonConfUtil.getConf("mqtt.broker") + ":" + CommonConfUtil.getConf("mqtt.port");
            String cliendId = careDevice.getClientId();
            String username = CommonConfUtil.getConf("mqtt.username");
            String password = CommonConfUtil.getConf("mqtt.password");
            String topic = careDevice.getTopic();
            configParams.put("MQTT_HOST", host);
            configParams.put("MQTT_CLIENTID", cliendId);
            configParams.put("MQTT_USERNAME", username);
            configParams.put("MQTT_PASSWORD", password);
            try {
                xbMqttDataConnector.createIotDataSource(configParams);
                MqttMessageListener mqttMessageListener = new MqttMessageListener();
                mqttMessageListener.setCareDeviceService(careDeviceService);
                mqttMessageListener.setBmsEventOrderService(bmsEventOrderService);
                mqttMessageListener.setCareEventOrderService(careEventOrderService);
                mqttMessageListener.setCareHouseService(careHouseService);
                mqttMessageListener.setCareOrganizationService(careOrganizationService);
                mqttMessageListener.setCareStationService(careStationService);

                xbMqttDataConnector.setReSubscribe(topic, mqttMessageListener);
                mqttDataConnectorMap.put(cliendId, xbMqttDataConnector);
            } catch (Exception e) {
                e.printStackTrace();
            }
        }
    }

    /**
     * 当设备表变更时需要重新初始化
     * @param careDeviceService
     * @param bmsEventOrderService
     * @param careEventOrderService
     * @param careHouseDeviceRelService
     * @param careHouseService
     * @param careOrganizationService
     * @param careStationService
     */
    public static void reInit(CareDeviceService careDeviceService,
                              BmsEventOrderService bmsEventOrderService,
                              CareEventOrderService careEventOrderService,
                              CareHouseService careHouseService,
                              CareOrganizationService careOrganizationService,
                              CareStationService careStationService) {
        mqttDataConnectorMap.forEach((key, value) -> {
            value.destroy();
            mqttDataConnectorMap.remove(key);
        });
        init(careDeviceService,bmsEventOrderService,careEventOrderService,careHouseService,careOrganizationService,careStationService);
    }

}
